from __future__ import absolute_import

import re  # noqa: F401

# python 2 and python 3 compatibility library
import six

from conductor.client.http.api_client import ApiClient


class EventResourceApi(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    Ref: https://github.com/swagger-api/swagger-codegen
    """

    def __init__(self, api_client=None):
        if api_client is None:
            api_client = ApiClient()
        self.api_client = api_client

    def add_event_handler(self, body, **kwargs):  # noqa: E501
        """Add a new event handler.  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.add_event_handler(body, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param EventHandler body: (required)
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.add_event_handler_with_http_info(body, **kwargs)  # noqa: E501
        else:
            (data) = self.add_event_handler_with_http_info(body, **kwargs)  # noqa: E501
            return data

    def add_event_handler_with_http_info(self, body, **kwargs):  # noqa: E501
        """Add a new event handler.  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.add_event_handler_with_http_info(body, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param EventHandler body: (required)
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['body']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method add_event_handler" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'body' is set
        if ('body' not in params or
                params['body'] is None):
            raise ValueError("Missing the required parameter `body` when calling `add_event_handler`")  # noqa: E501

        collection_formats = {}

        path_params = {}

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        if 'body' in params:
            body_params = params['body']
        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = []  # noqa: E501

        return self.api_client.call_api(
            '/event', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type=None,  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def delete_queue_config(self, queue_type, queue_name, **kwargs):  # noqa: E501
        """Delete queue config by name  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.delete_queue_config(queue_type, queue_name, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str queue_type: (required)
        :param str queue_name: (required)
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.delete_queue_config_with_http_info(queue_type, queue_name, **kwargs)  # noqa: E501
        else:
            (data) = self.delete_queue_config_with_http_info(queue_type, queue_name, **kwargs)  # noqa: E501
            return data

    def delete_queue_config_with_http_info(self, queue_type, queue_name, **kwargs):  # noqa: E501
        """Delete queue config by name  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.delete_queue_config_with_http_info(queue_type, queue_name, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str queue_type: (required)
        :param str queue_name: (required)
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['queue_type', 'queue_name']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method delete_queue_config" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'queue_type' is set
        if ('queue_type' not in params or
                params['queue_type'] is None):
            raise ValueError("Missing the required parameter `queue_type` when calling `delete_queue_config`")  # noqa: E501
        # verify the required parameter 'queue_name' is set
        if ('queue_name' not in params or
                params['queue_name'] is None):
            raise ValueError("Missing the required parameter `queue_name` when calling `delete_queue_config`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'queue_type' in params:
            path_params['queueType'] = params['queue_type']  # noqa: E501
        if 'queue_name' in params:
            path_params['queueName'] = params['queue_name']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # Authentication setting
        auth_settings = []  # noqa: E501

        return self.api_client.call_api(
            '/event/queue/config/{queueType}/{queueName}', 'DELETE',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type=None,  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def get_event_handlers(self, **kwargs):  # noqa: E501
        """Get all the event handlers  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_event_handlers(async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :return: list[EventHandler]
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.get_event_handlers_with_http_info(**kwargs)  # noqa: E501
        else:
            (data) = self.get_event_handlers_with_http_info(**kwargs)  # noqa: E501
            return data

    def get_event_handlers_with_http_info(self, **kwargs):  # noqa: E501
        """Get all the event handlers  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_event_handlers_with_http_info(async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :return: list[EventHandler]
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = []  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_event_handlers" % key
                )
            params[key] = val
        del params['kwargs']

        collection_formats = {}

        path_params = {}

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['*/*'])  # noqa: E501

        # Authentication setting
        auth_settings = []  # noqa: E501

        return self.api_client.call_api(
            '/event', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='list[EventHandler]',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def get_event_handlers_for_event(self, event, **kwargs):  # noqa: E501
        """Get event handlers for a given event  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_event_handlers_for_event(event, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str event: (required)
        :param bool active_only:
        :return: list[EventHandler]
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.get_event_handlers_for_event_with_http_info(event, **kwargs)  # noqa: E501
        else:
            (data) = self.get_event_handlers_for_event_with_http_info(event, **kwargs)  # noqa: E501
            return data

    def get_event_handlers_for_event_with_http_info(self, event, **kwargs):  # noqa: E501
        """Get event handlers for a given event  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_event_handlers_for_event_with_http_info(event, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str event: (required)
        :param bool active_only:
        :return: list[EventHandler]
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['event', 'active_only']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_event_handlers_for_event" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'event' is set
        if ('event' not in params or
                params['event'] is None):
            raise ValueError("Missing the required parameter `event` when calling `get_event_handlers_for_event`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'event' in params:
            path_params['event'] = params['event']  # noqa: E501

        query_params = []
        if 'active_only' in params:
            query_params.append(('activeOnly', params['active_only']))  # noqa: E501

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['*/*'])  # noqa: E501

        # Authentication setting
        auth_settings = []  # noqa: E501

        return self.api_client.call_api(
            '/event/{event}', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='list[EventHandler]',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def get_queue_config(self, queue_type, queue_name, **kwargs):  # noqa: E501
        """Get queue config by name  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_queue_config(queue_type, queue_name, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str queue_type: (required)
        :param str queue_name: (required)
        :return: dict(str, object)
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.get_queue_config_with_http_info(queue_type, queue_name, **kwargs)  # noqa: E501
        else:
            (data) = self.get_queue_config_with_http_info(queue_type, queue_name, **kwargs)  # noqa: E501
            return data

    def get_queue_config_with_http_info(self, queue_type, queue_name, **kwargs):  # noqa: E501
        """Get queue config by name  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_queue_config_with_http_info(queue_type, queue_name, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str queue_type: (required)
        :param str queue_name: (required)
        :return: dict(str, object)
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['queue_type', 'queue_name']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_queue_config" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'queue_type' is set
        if ('queue_type' not in params or
                params['queue_type'] is None):
            raise ValueError("Missing the required parameter `queue_type` when calling `get_queue_config`")  # noqa: E501
        # verify the required parameter 'queue_name' is set
        if ('queue_name' not in params or
                params['queue_name'] is None):
            raise ValueError("Missing the required parameter `queue_name` when calling `get_queue_config`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'queue_type' in params:
            path_params['queueType'] = params['queue_type']  # noqa: E501
        if 'queue_name' in params:
            path_params['queueName'] = params['queue_name']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['*/*'])  # noqa: E501

        # Authentication setting
        auth_settings = []  # noqa: E501

        return self.api_client.call_api(
            '/event/queue/config/{queueType}/{queueName}', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='dict(str, object)',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def get_queue_names(self, **kwargs):  # noqa: E501
        """Get all queue configs  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_queue_names(async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :return: dict(str, str)
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.get_queue_names_with_http_info(**kwargs)  # noqa: E501
        else:
            (data) = self.get_queue_names_with_http_info(**kwargs)  # noqa: E501
            return data

    def get_queue_names_with_http_info(self, **kwargs):  # noqa: E501
        """Get all queue configs  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_queue_names_with_http_info(async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :return: dict(str, str)
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = []  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_queue_names" % key
                )
            params[key] = val
        del params['kwargs']

        collection_formats = {}

        path_params = {}

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['*/*'])  # noqa: E501

        # Authentication setting
        auth_settings = []  # noqa: E501

        return self.api_client.call_api(
            '/event/queue/config', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='dict(str, str)',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def put_queue_config(self, body, queue_type, queue_name, **kwargs):  # noqa: E501
        """Create or update queue config by name  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.put_queue_config(body, queue_type, queue_name, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str body: (required)
        :param str queue_type: (required)
        :param str queue_name: (required)
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.put_queue_config_with_http_info(body, queue_type, queue_name, **kwargs)  # noqa: E501
        else:
            (data) = self.put_queue_config_with_http_info(body, queue_type, queue_name, **kwargs)  # noqa: E501
            return data

    def put_queue_config_with_http_info(self, body, queue_type, queue_name, **kwargs):  # noqa: E501
        """Create or update queue config by name  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.put_queue_config_with_http_info(body, queue_type, queue_name, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str body: (required)
        :param str queue_type: (required)
        :param str queue_name: (required)
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['body', 'queue_type', 'queue_name']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method put_queue_config" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'body' is set
        if ('body' not in params or
                params['body'] is None):
            raise ValueError("Missing the required parameter `body` when calling `put_queue_config`")  # noqa: E501
        # verify the required parameter 'queue_type' is set
        if ('queue_type' not in params or
                params['queue_type'] is None):
            raise ValueError("Missing the required parameter `queue_type` when calling `put_queue_config`")  # noqa: E501
        # verify the required parameter 'queue_name' is set
        if ('queue_name' not in params or
                params['queue_name'] is None):
            raise ValueError("Missing the required parameter `queue_name` when calling `put_queue_config`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'queue_type' in params:
            path_params['queueType'] = params['queue_type']  # noqa: E501
        if 'queue_name' in params:
            path_params['queueName'] = params['queue_name']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        if 'body' in params:
            body_params = params['body']
        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = []  # noqa: E501

        return self.api_client.call_api(
            '/event/queue/config/{queueType}/{queueName}', 'PUT',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type=None,  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def remove_event_handler_status(self, name, **kwargs):  # noqa: E501
        """Remove an event handler  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.remove_event_handler_status(name, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str name: (required)
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.remove_event_handler_status_with_http_info(name, **kwargs)  # noqa: E501
        else:
            (data) = self.remove_event_handler_status_with_http_info(name, **kwargs)  # noqa: E501
            return data

    def remove_event_handler_status_with_http_info(self, name, **kwargs):  # noqa: E501
        """Remove an event handler  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.remove_event_handler_status_with_http_info(name, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str name: (required)
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['name']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method remove_event_handler_status" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'name' is set
        if ('name' not in params or
                params['name'] is None):
            raise ValueError("Missing the required parameter `name` when calling `remove_event_handler_status`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'name' in params:
            path_params['name'] = params['name']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # Authentication setting
        auth_settings = []  # noqa: E501

        return self.api_client.call_api(
            '/event/{name}', 'DELETE',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type=None,  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def update_event_handler(self, body, **kwargs):  # noqa: E501
        """Update an existing event handler.  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.update_event_handler(body, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param EventHandler body: (required)
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.update_event_handler_with_http_info(body, **kwargs)  # noqa: E501
        else:
            (data) = self.update_event_handler_with_http_info(body, **kwargs)  # noqa: E501
            return data

    def update_event_handler_with_http_info(self, body, **kwargs):  # noqa: E501
        """Update an existing event handler.  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.update_event_handler_with_http_info(body, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param EventHandler body: (required)
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['body']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method update_event_handler" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'body' is set
        if ('body' not in params or
                params['body'] is None):
            raise ValueError("Missing the required parameter `body` when calling `update_event_handler`")  # noqa: E501

        collection_formats = {}

        path_params = {}

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        if 'body' in params:
            body_params = params['body']
        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = []  # noqa: E501

        return self.api_client.call_api(
            '/event', 'PUT',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type=None,  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)



from conductor.client.automator.task_runner import TaskRunner
from conductor.client.configuration.configuration import Configuration
from conductor.client.configuration.settings.metrics_settings import MetricsSettings
from conductor.client.telemetry.metrics_collector import MetricsCollector
from conductor.client.worker.worker_interface import WorkerInterface
from multiprocessing import Process
from typing import List
import logging

logger = logging.getLogger(
    Configuration.get_logging_formatted_name(
        __name__
    )
)


class TaskHandler:
    def __init__(
            self,
            workers: List[WorkerInterface],
            configuration: Configuration = None,
            metrics_settings: MetricsSettings = None
    ):
        if not isinstance(workers, list):
            workers = [workers]
        self.__create_task_runner_processes(
            workers, configuration, metrics_settings
        )
        self.__create_metrics_provider_process(
            metrics_settings
        )
        logger.info('Created all processes')

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        self.stop_processes()

    def stop_processes(self) -> None:
        self.__stop_task_runner_processes()
        self.__stop_metrics_provider_process()
        logger.debug('stopped processes')

    def start_processes(self) -> None:
        self.__start_task_runner_processes()
        self.__start_metrics_provider_process()
        logger.info('Started all processes')

    def join_processes(self) -> None:
        self.__join_task_runner_processes()
        self.__join_metrics_provider_process()
        logger.info('Joined all processes')

    def __create_metrics_provider_process(self, metrics_settings: MetricsSettings) -> None:
        if metrics_settings == None:
            self.metrics_provider_process = None
            return
        self.metrics_provider_process = Process(
            target=MetricsCollector.provide_metrics,
            args=(metrics_settings,)
        )
        logger.info('Created MetricsProvider process')

    def __create_task_runner_processes(
        self,
        workers: List[WorkerInterface],
        configuration: Configuration,
        metrics_settings: MetricsSettings
    ) -> None:
        self.task_runner_processes = []
        for worker in workers:
            self.__create_task_runner_process(
                worker, configuration, metrics_settings
            )
        logger.info('Created TaskRunner processes')

    def __create_task_runner_process(
        self,
        worker: WorkerInterface,
        configuration: Configuration,
        metrics_settings: MetricsSettings
    ) -> None:
        task_runner = TaskRunner(worker, configuration, metrics_settings)
        process = Process(
            target=task_runner.run
        )
        self.task_runner_processes.append(process)

    def __start_metrics_provider_process(self):
        if self.metrics_provider_process == None:
            return
        self.metrics_provider_process.start()
        logger.info('Started MetricsProvider process')

    def __start_task_runner_processes(self):
        for task_runner_process in self.task_runner_processes:
            task_runner_process.start()
        logger.info('Started TaskRunner processes')

    def __join_metrics_provider_process(self):
        if self.metrics_provider_process == None:
            return
        self.metrics_provider_process.join()
        logger.info('Joined MetricsProvider processes')

    def __join_task_runner_processes(self):
        for task_runner_process in self.task_runner_processes:
            task_runner_process.join()
        logger.info('Joined TaskRunner processes')

    def __stop_metrics_provider_process(self):
        self.__stop_process(self.metrics_provider_process)

    def __stop_task_runner_processes(self):
        for task_runner_process in self.task_runner_processes:
            self.__stop_process(task_runner_process)

    def __stop_process(self, process: Process):
        if process == None:
            return
        try:
            process.kill()
            logger.debug(f'Killed process: {process}')
        except Exception as e:
            logger.debug(f'Failed to kill process: {process}, reason: {e}')
            process.terminate()
            logger.debug('Terminated process: {process}')



from conductor.client.workflow.task.task import TaskInterface
from conductor.client.workflow.task.task_type import TaskType
from copy import deepcopy
from enum import Enum
from typing import Any, Dict, List
from typing_extensions import Self


class HttpMethod(str, Enum):
    GET = "GET",
    PUT = "PUT",
    POST = "POST",
    DELETE = "DELETE",
    HEAD = "HEAD",
    OPTIONS = "OPTIONS"


class HttpInput:
    swagger_types = {
        '_uri': 'str',
        '_method': 'str',
        '_accept': 'list[str]',
        '_headers': 'dict[str, list[str]]',
        '_accept': 'str',
        '_content_type': 'str',
        '_connection_time_out': 'int',
        '_read_timeout': 'int',
    }

    attribute_map = {
        '_uri': 'uri',
        '_method': 'method',
        '_accept': 'accept',
        '_headers': 'headers',
        '_accept': 'accept',
        '_content_type': 'contentType',
        '_connection_time_out': 'connectionTimeOut',
        '_read_timeout': 'readTimeOut',
    }

    def __init__(self,
                 method: HttpMethod = HttpMethod.GET,
                 uri: str = None,
                 headers: Dict[str, List[str]] = None,
                 accept: str = None,
                 content_type: str = None,
                 connection_time_out: int = None,
                 read_timeout: int = None,
                 body: Any = None) -> Self:
        self._method = deepcopy(method)
        self._uri = deepcopy(uri)
        self._headers = deepcopy(headers)
        self._accept = deepcopy(accept)
        self._content_type = deepcopy(content_type)
        self._connection_time_out = deepcopy(connection_time_out)
        self._read_timeout = deepcopy(read_timeout)
        self._body = deepcopy(body)


class HttpTask(TaskInterface):
    def __init__(self, task_ref_name: str, http_input: HttpInput) -> Self:
        super().__init__(
            task_reference_name=task_ref_name,
            task_type=TaskType.HTTP,
            input_parameters={
                "http_request": http_input
            }
        )



import pprint
import re  # noqa: F401

import six

class PollData(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'queue_name': 'str',
        'domain': 'str',
        'worker_id': 'str',
        'last_poll_time': 'int'
    }

    attribute_map = {
        'queue_name': 'queueName',
        'domain': 'domain',
        'worker_id': 'workerId',
        'last_poll_time': 'lastPollTime'
    }

    def __init__(self, queue_name=None, domain=None, worker_id=None, last_poll_time=None):  # noqa: E501
        """PollData - a model defined in Swagger"""  # noqa: E501
        self._queue_name = None
        self._domain = None
        self._worker_id = None
        self._last_poll_time = None
        self.discriminator = None
        if queue_name is not None:
            self.queue_name = queue_name
        if domain is not None:
            self.domain = domain
        if worker_id is not None:
            self.worker_id = worker_id
        if last_poll_time is not None:
            self.last_poll_time = last_poll_time

    @property
    def queue_name(self):
        """Gets the queue_name of this PollData.  # noqa: E501


        :return: The queue_name of this PollData.  # noqa: E501
        :rtype: str
        """
        return self._queue_name

    @queue_name.setter
    def queue_name(self, queue_name):
        """Sets the queue_name of this PollData.


        :param queue_name: The queue_name of this PollData.  # noqa: E501
        :type: str
        """

        self._queue_name = queue_name

    @property
    def domain(self):
        """Gets the domain of this PollData.  # noqa: E501


        :return: The domain of this PollData.  # noqa: E501
        :rtype: str
        """
        return self._domain

    @domain.setter
    def domain(self, domain):
        """Sets the domain of this PollData.


        :param domain: The domain of this PollData.  # noqa: E501
        :type: str
        """

        self._domain = domain

    @property
    def worker_id(self):
        """Gets the worker_id of this PollData.  # noqa: E501


        :return: The worker_id of this PollData.  # noqa: E501
        :rtype: str
        """
        return self._worker_id

    @worker_id.setter
    def worker_id(self, worker_id):
        """Sets the worker_id of this PollData.


        :param worker_id: The worker_id of this PollData.  # noqa: E501
        :type: str
        """

        self._worker_id = worker_id

    @property
    def last_poll_time(self):
        """Gets the last_poll_time of this PollData.  # noqa: E501


        :return: The last_poll_time of this PollData.  # noqa: E501
        :rtype: int
        """
        return self._last_poll_time

    @last_poll_time.setter
    def last_poll_time(self, last_poll_time):
        """Sets the last_poll_time of this PollData.


        :param last_poll_time: The last_poll_time of this PollData.  # noqa: E501
        :type: int
        """

        self._last_poll_time = last_poll_time

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(PollData, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, PollData):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



from conductor.client.http.models.task_result_status import TaskResultStatus
import pprint
import re  # noqa: F401
import six


class TaskResult(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'workflow_instance_id': 'str',
        'task_id': 'str',
        'reason_for_incompletion': 'str',
        'callback_after_seconds': 'int',
        'worker_id': 'str',
        'status': 'str',
        'output_data': 'dict(str, object)',
        'logs': 'list[TaskExecLog]',
        'external_output_payload_storage_path': 'str',
        'sub_workflow_id': 'str'
    }

    attribute_map = {
        'workflow_instance_id': 'workflowInstanceId',
        'task_id': 'taskId',
        'reason_for_incompletion': 'reasonForIncompletion',
        'callback_after_seconds': 'callbackAfterSeconds',
        'worker_id': 'workerId',
        'status': 'status',
        'output_data': 'outputData',
        'logs': 'logs',
        'external_output_payload_storage_path': 'externalOutputPayloadStoragePath',
        'sub_workflow_id': 'subWorkflowId'
    }

    def __init__(self, workflow_instance_id=None, task_id=None, reason_for_incompletion=None, callback_after_seconds=None, worker_id=None, status=None, output_data=None, logs=None, external_output_payload_storage_path=None, sub_workflow_id=None):  # noqa: E501
        """TaskResult - a model defined in Swagger"""  # noqa: E501
        self._workflow_instance_id = None
        self._task_id = None
        self._reason_for_incompletion = None
        self._callback_after_seconds = None
        self._worker_id = None
        self._status = None
        self._output_data = None
        self._logs = None
        self._external_output_payload_storage_path = None
        self._sub_workflow_id = None
        self.discriminator = None
        self.workflow_instance_id = workflow_instance_id
        self.task_id = task_id
        if reason_for_incompletion is not None:
            self.reason_for_incompletion = reason_for_incompletion
        if callback_after_seconds is not None:
            self.callback_after_seconds = callback_after_seconds
        if worker_id is not None:
            self.worker_id = worker_id
        if status is not None:
            self.status = status
        if output_data is not None:
            self.output_data = output_data
        if logs is not None:
            self.logs = logs
        if external_output_payload_storage_path is not None:
            self.external_output_payload_storage_path = external_output_payload_storage_path
        if sub_workflow_id is not None:
            self.sub_workflow_id = sub_workflow_id

    @property
    def workflow_instance_id(self):
        """Gets the workflow_instance_id of this TaskResult.  # noqa: E501


        :return: The workflow_instance_id of this TaskResult.  # noqa: E501
        :rtype: str
        """
        return self._workflow_instance_id

    @workflow_instance_id.setter
    def workflow_instance_id(self, workflow_instance_id):
        """Sets the workflow_instance_id of this TaskResult.


        :param workflow_instance_id: The workflow_instance_id of this TaskResult.  # noqa: E501
        :type: str
        """
        self._workflow_instance_id = workflow_instance_id

    @property
    def task_id(self):
        """Gets the task_id of this TaskResult.  # noqa: E501


        :return: The task_id of this TaskResult.  # noqa: E501
        :rtype: str
        """
        return self._task_id

    @task_id.setter
    def task_id(self, task_id):
        """Sets the task_id of this TaskResult.


        :param task_id: The task_id of this TaskResult.  # noqa: E501
        :type: str
        """
        self._task_id = task_id

    @property
    def reason_for_incompletion(self):
        """Gets the reason_for_incompletion of this TaskResult.  # noqa: E501


        :return: The reason_for_incompletion of this TaskResult.  # noqa: E501
        :rtype: str
        """
        return self._reason_for_incompletion

    @reason_for_incompletion.setter
    def reason_for_incompletion(self, reason_for_incompletion):
        """Sets the reason_for_incompletion of this TaskResult.


        :param reason_for_incompletion: The reason_for_incompletion of this TaskResult.  # noqa: E501
        :type: str
        """

        self._reason_for_incompletion = reason_for_incompletion

    @property
    def callback_after_seconds(self):
        """Gets the callback_after_seconds of this TaskResult.  # noqa: E501


        :return: The callback_after_seconds of this TaskResult.  # noqa: E501
        :rtype: int
        """
        return self._callback_after_seconds

    @callback_after_seconds.setter
    def callback_after_seconds(self, callback_after_seconds):
        """Sets the callback_after_seconds of this TaskResult.


        :param callback_after_seconds: The callback_after_seconds of this TaskResult.  # noqa: E501
        :type: int
        """

        self._callback_after_seconds = callback_after_seconds

    @property
    def worker_id(self):
        """Gets the worker_id of this TaskResult.  # noqa: E501


        :return: The worker_id of this TaskResult.  # noqa: E501
        :rtype: str
        """
        return self._worker_id

    @worker_id.setter
    def worker_id(self, worker_id):
        """Sets the worker_id of this TaskResult.


        :param worker_id: The worker_id of this TaskResult.  # noqa: E501
        :type: str
        """

        self._worker_id = worker_id

    @property
    def status(self):
        """Gets the status of this TaskResult.  # noqa: E501


        :return: The status of this TaskResult.  # noqa: E501
        :rtype: str
        """
        return self._status

    @status.setter
    def status(self, status):
        """Sets the status of this TaskResult.


        :param status: The status of this TaskResult.  # noqa: E501
        :type: str
        """
        allowed_values = [
            task_result_status.name for task_result_status in TaskResultStatus
        ]
        if status not in allowed_values:
            raise ValueError(
                "Invalid value for `status` ({0}), must be one of {1}"  # noqa: E501
                .format(status, allowed_values)
            )

        self._status = TaskResultStatus[status]

    @property
    def output_data(self):
        """Gets the output_data of this TaskResult.  # noqa: E501


        :return: The output_data of this TaskResult.  # noqa: E501
        :rtype: dict(str, object)
        """
        return self._output_data

    @output_data.setter
    def output_data(self, output_data):
        """Sets the output_data of this TaskResult.


        :param output_data: The output_data of this TaskResult.  # noqa: E501
        :type: dict(str, object)
        """

        self._output_data = output_data

    @property
    def logs(self):
        """Gets the logs of this TaskResult.  # noqa: E501


        :return: The logs of this TaskResult.  # noqa: E501
        :rtype: list[TaskExecLog]
        """
        return self._logs

    @logs.setter
    def logs(self, logs):
        """Sets the logs of this TaskResult.


        :param logs: The logs of this TaskResult.  # noqa: E501
        :type: list[TaskExecLog]
        """

        self._logs = logs

    @property
    def external_output_payload_storage_path(self):
        """Gets the external_output_payload_storage_path of this TaskResult.  # noqa: E501


        :return: The external_output_payload_storage_path of this TaskResult.  # noqa: E501
        :rtype: str
        """
        return self._external_output_payload_storage_path

    @external_output_payload_storage_path.setter
    def external_output_payload_storage_path(self, external_output_payload_storage_path):
        """Sets the external_output_payload_storage_path of this TaskResult.


        :param external_output_payload_storage_path: The external_output_payload_storage_path of this TaskResult.  # noqa: E501
        :type: str
        """

        self._external_output_payload_storage_path = external_output_payload_storage_path

    @property
    def sub_workflow_id(self):
        """Gets the sub_workflow_id of this TaskResult.  # noqa: E501


        :return: The sub_workflow_id of this TaskResult.  # noqa: E501
        :rtype: str
        """
        return self._sub_workflow_id

    @sub_workflow_id.setter
    def sub_workflow_id(self, sub_workflow_id):
        """Sets the sub_workflow_id of this TaskResult.


        :param sub_workflow_id: The sub_workflow_id of this TaskResult.  # noqa: E501
        :type: str
        """

        self._sub_workflow_id = sub_workflow_id

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(TaskResult, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, TaskResult):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other

    def add_output_data(self, key, value):
        if self.output_data == None:
            self.output_data = {}
        self.output_data[key] = value



from conductor.client.http.models.start_workflow_request import StartWorkflowRequest
from conductor.client.workflow.task.task import TaskInterface
from conductor.client.workflow.task.task_type import TaskType
from typing_extensions import Self


class StartWorkflowTask(TaskInterface):
    def __init__(self, task_ref_name: str, workflow_name: str, start_workflow_request: StartWorkflowRequest, version: int = None) -> Self:
        super().__init__(
            task_reference_name=task_ref_name,
            task_type=TaskType.START_WORKFLOW,
            input_parameters={
                "startWorkflow": {
                    "name":          workflow_name,
                    "version":       version,
                    "input":         start_workflow_request.input,
                    "correlationId": start_workflow_request.correlation_id,
                },
            }
        )



from conductor.client.http.models.action import Action
from conductor.client.http.models.authorization_request import AuthorizationRequest
from conductor.client.http.models.bulk_response import BulkResponse
from conductor.client.http.models.conductor_application import ConductorApplication
from conductor.client.http.models.conductor_user import ConductorUser
from conductor.client.http.models.create_or_update_application_request import CreateOrUpdateApplicationRequest
from conductor.client.http.models.event_handler import EventHandler
from conductor.client.http.models.external_storage_location import ExternalStorageLocation
from conductor.client.http.models.generate_token_request import GenerateTokenRequest
from conductor.client.http.models.group import Group
from conductor.client.http.models.permission import Permission
from conductor.client.http.models.poll_data import PollData
from conductor.client.http.models.rate_limit import RateLimit
from conductor.client.http.models.rerun_workflow_request import RerunWorkflowRequest
from conductor.client.http.models.response import Response
from conductor.client.http.models.role import Role
from conductor.client.http.models.save_schedule_request import SaveScheduleRequest
from conductor.client.http.models.scrollable_search_result_workflow_summary import ScrollableSearchResultWorkflowSummary
from conductor.client.http.models.search_result_task import SearchResultTask
from conductor.client.http.models.search_result_task_summary import SearchResultTaskSummary
from conductor.client.http.models.search_result_workflow import SearchResultWorkflow
from conductor.client.http.models.search_result_workflow_schedule_execution_model import SearchResultWorkflowScheduleExecutionModel
from conductor.client.http.models.search_result_workflow_summary import SearchResultWorkflowSummary
from conductor.client.http.models.skip_task_request import SkipTaskRequest
from conductor.client.http.models.start_workflow import StartWorkflow
from conductor.client.http.models.start_workflow_request import StartWorkflowRequest
from conductor.client.http.models.sub_workflow_params import SubWorkflowParams
from conductor.client.http.models.subject_ref import SubjectRef
from conductor.client.http.models.target_ref import TargetRef
from conductor.client.http.models.task import Task
from conductor.client.http.models.task_def import TaskDef
from conductor.client.http.models.task_details import TaskDetails
from conductor.client.http.models.task_exec_log import TaskExecLog
from conductor.client.http.models.task_result import TaskResult
from conductor.client.http.models.task_summary import TaskSummary
from conductor.client.http.models.token import Token
from conductor.client.http.models.upsert_group_request import UpsertGroupRequest
from conductor.client.http.models.upsert_user_request import UpsertUserRequest
from conductor.client.http.models.workflow import Workflow
from conductor.client.http.models.workflow_def import WorkflowDef
from conductor.client.http.models.workflow_run import WorkflowRun
from conductor.client.http.models.workflow_schedule import WorkflowSchedule
from conductor.client.http.models.workflow_schedule_execution_model import WorkflowScheduleExecutionModel
from conductor.client.http.models.workflow_status import WorkflowStatus
from conductor.client.http.models.workflow_summary import WorkflowSummary
from conductor.client.http.models.workflow_tag import WorkflowTag
from conductor.client.http.models.workflow_task import WorkflowTask



import pprint
import re  # noqa: F401

import six

class HealthCheckStatus(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'health_results': 'list[Health]',
        'suppressed_health_results': 'list[Health]',
        'healthy': 'bool'
    }

    attribute_map = {
        'health_results': 'healthResults',
        'suppressed_health_results': 'suppressedHealthResults',
        'healthy': 'healthy'
    }

    def __init__(self, health_results=None, suppressed_health_results=None, healthy=None):  # noqa: E501
        """HealthCheckStatus - a model defined in Swagger"""  # noqa: E501
        self._health_results = None
        self._suppressed_health_results = None
        self._healthy = None
        self.discriminator = None
        if health_results is not None:
            self.health_results = health_results
        if suppressed_health_results is not None:
            self.suppressed_health_results = suppressed_health_results
        if healthy is not None:
            self.healthy = healthy

    @property
    def health_results(self):
        """Gets the health_results of this HealthCheckStatus.  # noqa: E501


        :return: The health_results of this HealthCheckStatus.  # noqa: E501
        :rtype: list[Health]
        """
        return self._health_results

    @health_results.setter
    def health_results(self, health_results):
        """Sets the health_results of this HealthCheckStatus.


        :param health_results: The health_results of this HealthCheckStatus.  # noqa: E501
        :type: list[Health]
        """

        self._health_results = health_results

    @property
    def suppressed_health_results(self):
        """Gets the suppressed_health_results of this HealthCheckStatus.  # noqa: E501


        :return: The suppressed_health_results of this HealthCheckStatus.  # noqa: E501
        :rtype: list[Health]
        """
        return self._suppressed_health_results

    @suppressed_health_results.setter
    def suppressed_health_results(self, suppressed_health_results):
        """Sets the suppressed_health_results of this HealthCheckStatus.


        :param suppressed_health_results: The suppressed_health_results of this HealthCheckStatus.  # noqa: E501
        :type: list[Health]
        """

        self._suppressed_health_results = suppressed_health_results

    @property
    def healthy(self):
        """Gets the healthy of this HealthCheckStatus.  # noqa: E501


        :return: The healthy of this HealthCheckStatus.  # noqa: E501
        :rtype: bool
        """
        return self._healthy

    @healthy.setter
    def healthy(self, healthy):
        """Sets the healthy of this HealthCheckStatus.


        :param healthy: The healthy of this HealthCheckStatus.  # noqa: E501
        :type: bool
        """

        self._healthy = healthy

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(HealthCheckStatus, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, HealthCheckStatus):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



from enum import Enum


class TaskResultStatus(str, Enum):
    COMPLETED = "COMPLETED",
    FAILED = "FAILED",
    FAILED_WITH_TERMINAL_ERROR = "FAILED_WITH_TERMINAL_ERROR",
    IN_PROGRESS = "IN_PROGRESS"



import pprint
import re  # noqa: F401

import six

class SubWorkflowParams(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'name': 'str',
        'version': 'int',
        'task_to_domain': 'dict(str, str)',
        'workflow_definition': 'WorkflowDef'
    }

    attribute_map = {
        'name': 'name',
        'version': 'version',
        'task_to_domain': 'taskToDomain',
        'workflow_definition': 'workflowDefinition'
    }

    def __init__(self, name=None, version=None, task_to_domain=None, workflow_definition=None):  # noqa: E501
        """SubWorkflowParams - a model defined in Swagger"""  # noqa: E501
        self._name = None
        self._version = None
        self._task_to_domain = None
        self._workflow_definition = None
        self.discriminator = None
        self.name = name
        if version is not None:
            self.version = version
        if task_to_domain is not None:
            self.task_to_domain = task_to_domain
        if workflow_definition is not None:
            self.workflow_definition = workflow_definition

    @property
    def name(self):
        """Gets the name of this SubWorkflowParams.  # noqa: E501


        :return: The name of this SubWorkflowParams.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this SubWorkflowParams.


        :param name: The name of this SubWorkflowParams.  # noqa: E501
        :type: str
        """
        self._name = name

    @property
    def version(self):
        """Gets the version of this SubWorkflowParams.  # noqa: E501


        :return: The version of this SubWorkflowParams.  # noqa: E501
        :rtype: int
        """
        return self._version

    @version.setter
    def version(self, version):
        """Sets the version of this SubWorkflowParams.


        :param version: The version of this SubWorkflowParams.  # noqa: E501
        :type: int
        """

        self._version = version

    @property
    def task_to_domain(self):
        """Gets the task_to_domain of this SubWorkflowParams.  # noqa: E501


        :return: The task_to_domain of this SubWorkflowParams.  # noqa: E501
        :rtype: dict(str, str)
        """
        return self._task_to_domain

    @task_to_domain.setter
    def task_to_domain(self, task_to_domain):
        """Sets the task_to_domain of this SubWorkflowParams.


        :param task_to_domain: The task_to_domain of this SubWorkflowParams.  # noqa: E501
        :type: dict(str, str)
        """

        self._task_to_domain = task_to_domain

    @property
    def workflow_definition(self):
        """Gets the workflow_definition of this SubWorkflowParams.  # noqa: E501


        :return: The workflow_definition of this SubWorkflowParams.  # noqa: E501
        :rtype: WorkflowDef
        """
        return self._workflow_definition

    @workflow_definition.setter
    def workflow_definition(self, workflow_definition):
        """Sets the workflow_definition of this SubWorkflowParams.


        :param workflow_definition: The workflow_definition of this SubWorkflowParams.  # noqa: E501
        :type: WorkflowDef
        """

        self._workflow_definition = workflow_definition

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(SubWorkflowParams, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, SubWorkflowParams):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



import pprint
import re  # noqa: F401

import six

class WorkflowRun(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'correlation_id': 'str',
        'create_time': 'int',
        'created_by': 'str',
        'input': 'dict(str, object)',
        'output': 'dict(str, object)',
        'priority': 'int',
        'request_id': 'str',
        'status': 'str',
        'tasks': 'list[Task]',
        'update_time': 'int',
        'variables': 'dict(str, object)',
        'workflow_id': 'str'
    }

    attribute_map = {
        'correlation_id': 'correlationId',
        'create_time': 'createTime',
        'created_by': 'createdBy',
        'input': 'input',
        'output': 'output',
        'priority': 'priority',
        'request_id': 'requestId',
        'status': 'status',
        'tasks': 'tasks',
        'update_time': 'updateTime',
        'variables': 'variables',
        'workflow_id': 'workflowId'
    }

    def __init__(self, correlation_id=None, create_time=None, created_by=None, input=None, output=None, priority=None, request_id=None, status=None, tasks=None, update_time=None, variables=None, workflow_id=None):  # noqa: E501
        """WorkflowRun - a model defined in Swagger"""  # noqa: E501
        self._correlation_id = None
        self._create_time = None
        self._created_by = None
        self._input = None
        self._output = None
        self._priority = None
        self._request_id = None
        self._status = None
        self._tasks = None
        self._update_time = None
        self._variables = None
        self._workflow_id = None
        self.discriminator = None
        if correlation_id is not None:
            self.correlation_id = correlation_id
        if create_time is not None:
            self.create_time = create_time
        if created_by is not None:
            self.created_by = created_by
        if input is not None:
            self.input = input
        if output is not None:
            self.output = output
        if priority is not None:
            self.priority = priority
        if request_id is not None:
            self.request_id = request_id
        if status is not None:
            self.status = status
        if tasks is not None:
            self.tasks = tasks
        if update_time is not None:
            self.update_time = update_time
        if variables is not None:
            self.variables = variables
        if workflow_id is not None:
            self.workflow_id = workflow_id

    @property
    def correlation_id(self):
        """Gets the correlation_id of this WorkflowRun.  # noqa: E501


        :return: The correlation_id of this WorkflowRun.  # noqa: E501
        :rtype: str
        """
        return self._correlation_id

    @correlation_id.setter
    def correlation_id(self, correlation_id):
        """Sets the correlation_id of this WorkflowRun.


        :param correlation_id: The correlation_id of this WorkflowRun.  # noqa: E501
        :type: str
        """

        self._correlation_id = correlation_id

    @property
    def create_time(self):
        """Gets the create_time of this WorkflowRun.  # noqa: E501


        :return: The create_time of this WorkflowRun.  # noqa: E501
        :rtype: int
        """
        return self._create_time

    @create_time.setter
    def create_time(self, create_time):
        """Sets the create_time of this WorkflowRun.


        :param create_time: The create_time of this WorkflowRun.  # noqa: E501
        :type: int
        """

        self._create_time = create_time

    @property
    def created_by(self):
        """Gets the created_by of this WorkflowRun.  # noqa: E501


        :return: The created_by of this WorkflowRun.  # noqa: E501
        :rtype: str
        """
        return self._created_by

    @created_by.setter
    def created_by(self, created_by):
        """Sets the created_by of this WorkflowRun.


        :param created_by: The created_by of this WorkflowRun.  # noqa: E501
        :type: str
        """

        self._created_by = created_by

    @property
    def input(self):
        """Gets the input of this WorkflowRun.  # noqa: E501


        :return: The input of this WorkflowRun.  # noqa: E501
        :rtype: dict(str, object)
        """
        return self._input

    @input.setter
    def input(self, input):
        """Sets the input of this WorkflowRun.


        :param input: The input of this WorkflowRun.  # noqa: E501
        :type: dict(str, object)
        """

        self._input = input

    @property
    def output(self):
        """Gets the output of this WorkflowRun.  # noqa: E501


        :return: The output of this WorkflowRun.  # noqa: E501
        :rtype: dict(str, object)
        """
        return self._output

    @output.setter
    def output(self, output):
        """Sets the output of this WorkflowRun.


        :param output: The output of this WorkflowRun.  # noqa: E501
        :type: dict(str, object)
        """

        self._output = output

    @property
    def priority(self):
        """Gets the priority of this WorkflowRun.  # noqa: E501


        :return: The priority of this WorkflowRun.  # noqa: E501
        :rtype: int
        """
        return self._priority

    @priority.setter
    def priority(self, priority):
        """Sets the priority of this WorkflowRun.


        :param priority: The priority of this WorkflowRun.  # noqa: E501
        :type: int
        """

        self._priority = priority

    @property
    def request_id(self):
        """Gets the request_id of this WorkflowRun.  # noqa: E501


        :return: The request_id of this WorkflowRun.  # noqa: E501
        :rtype: str
        """
        return self._request_id

    @request_id.setter
    def request_id(self, request_id):
        """Sets the request_id of this WorkflowRun.


        :param request_id: The request_id of this WorkflowRun.  # noqa: E501
        :type: str
        """

        self._request_id = request_id

    @property
    def status(self):
        """Gets the status of this WorkflowRun.  # noqa: E501


        :return: The status of this WorkflowRun.  # noqa: E501
        :rtype: str
        """
        return self._status

    @status.setter
    def status(self, status):
        """Sets the status of this WorkflowRun.


        :param status: The status of this WorkflowRun.  # noqa: E501
        :type: str
        """
        allowed_values = ["RUNNING", "COMPLETED", "FAILED", "TIMED_OUT", "TERMINATED", "PAUSED"]  # noqa: E501
        if status not in allowed_values:
            raise ValueError(
                "Invalid value for `status` ({0}), must be one of {1}"  # noqa: E501
                .format(status, allowed_values)
            )

        self._status = status

    @property
    def tasks(self):
        """Gets the tasks of this WorkflowRun.  # noqa: E501


        :return: The tasks of this WorkflowRun.  # noqa: E501
        :rtype: list[Task]
        """
        return self._tasks

    @tasks.setter
    def tasks(self, tasks):
        """Sets the tasks of this WorkflowRun.


        :param tasks: The tasks of this WorkflowRun.  # noqa: E501
        :type: list[Task]
        """

        self._tasks = tasks

    @property
    def update_time(self):
        """Gets the update_time of this WorkflowRun.  # noqa: E501


        :return: The update_time of this WorkflowRun.  # noqa: E501
        :rtype: int
        """
        return self._update_time

    @update_time.setter
    def update_time(self, update_time):
        """Sets the update_time of this WorkflowRun.


        :param update_time: The update_time of this WorkflowRun.  # noqa: E501
        :type: int
        """

        self._update_time = update_time

    @property
    def variables(self):
        """Gets the variables of this WorkflowRun.  # noqa: E501


        :return: The variables of this WorkflowRun.  # noqa: E501
        :rtype: dict(str, object)
        """
        return self._variables

    @variables.setter
    def variables(self, variables):
        """Sets the variables of this WorkflowRun.


        :param variables: The variables of this WorkflowRun.  # noqa: E501
        :type: dict(str, object)
        """

        self._variables = variables

    @property
    def workflow_id(self):
        """Gets the workflow_id of this WorkflowRun.  # noqa: E501


        :return: The workflow_id of this WorkflowRun.  # noqa: E501
        :rtype: str
        """
        return self._workflow_id

    @workflow_id.setter
    def workflow_id(self, workflow_id):
        """Sets the workflow_id of this WorkflowRun.


        :param workflow_id: The workflow_id of this WorkflowRun.  # noqa: E501
        :type: str
        """

        self._workflow_id = workflow_id

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(WorkflowRun, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, WorkflowRun):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



from conductor.client.http.models.sub_workflow_params import SubWorkflowParams
from conductor.client.http.models.workflow_task import WorkflowTask
from conductor.client.workflow.conductor_workflow import ConductorWorkflow
from conductor.client.workflow.task.task import TaskInterface
from conductor.client.workflow.task.task_type import TaskType
from copy import deepcopy
from typing import Dict
from typing_extensions import Self


class SubWorkflowTask(TaskInterface):
    def __init__(self, task_ref_name: str, workflow_name: str, version: int = None, task_to_domain_map: Dict[str, str] = None) -> Self:
        super().__init__(
            task_reference_name=task_ref_name,
            task_type=TaskType.SUB_WORKFLOW
        )
        self._workflow_name = deepcopy(workflow_name)
        self._version = deepcopy(version)
        self._task_to_domain_map = deepcopy(task_to_domain_map)

    def to_workflow_task(self) -> WorkflowTask:
        workflow = super().to_workflow_task()
        workflow.sub_workflow_param = SubWorkflowParams(
            name=self._workflow_name,
            version=self._version,
            task_to_domain=self._task_to_domain_map,
        )
        return workflow


class InlineSubWorkflowTask(TaskInterface):
    def __init__(self, task_ref_name: str, workflow: ConductorWorkflow) -> Self:
        super().__init__(
            task_reference_name=task_ref_name,
            task_type=TaskType.SUB_WORKFLOW,
        )
        self._workflow_name = deepcopy(workflow.name)
        self._workflow_version = deepcopy(workflow.version)
        self._workflow_definition = deepcopy(workflow.to_workflow_def())

    def to_workflow_task(self) -> WorkflowTask:
        workflow = super().to_workflow_task()
        workflow.sub_workflow_param = SubWorkflowParams(
            name=self._workflow_name,
            version=self._workflow_version,
            workflow_definition=self._workflow_definition,
        )
        return workflow



import pprint
import re  # noqa: F401

import six

class RerunWorkflowRequest(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        're_run_from_workflow_id': 'str',
        'workflow_input': 'dict(str, object)',
        're_run_from_task_id': 'str',
        'task_input': 'dict(str, object)',
        'correlation_id': 'str'
    }

    attribute_map = {
        're_run_from_workflow_id': 'reRunFromWorkflowId',
        'workflow_input': 'workflowInput',
        're_run_from_task_id': 'reRunFromTaskId',
        'task_input': 'taskInput',
        'correlation_id': 'correlationId'
    }

    def __init__(self, re_run_from_workflow_id=None, workflow_input=None, re_run_from_task_id=None, task_input=None, correlation_id=None):  # noqa: E501
        """RerunWorkflowRequest - a model defined in Swagger"""  # noqa: E501
        self._re_run_from_workflow_id = None
        self._workflow_input = None
        self._re_run_from_task_id = None
        self._task_input = None
        self._correlation_id = None
        self.discriminator = None
        if re_run_from_workflow_id is not None:
            self.re_run_from_workflow_id = re_run_from_workflow_id
        if workflow_input is not None:
            self.workflow_input = workflow_input
        if re_run_from_task_id is not None:
            self.re_run_from_task_id = re_run_from_task_id
        if task_input is not None:
            self.task_input = task_input
        if correlation_id is not None:
            self.correlation_id = correlation_id

    @property
    def re_run_from_workflow_id(self):
        """Gets the re_run_from_workflow_id of this RerunWorkflowRequest.  # noqa: E501


        :return: The re_run_from_workflow_id of this RerunWorkflowRequest.  # noqa: E501
        :rtype: str
        """
        return self._re_run_from_workflow_id

    @re_run_from_workflow_id.setter
    def re_run_from_workflow_id(self, re_run_from_workflow_id):
        """Sets the re_run_from_workflow_id of this RerunWorkflowRequest.


        :param re_run_from_workflow_id: The re_run_from_workflow_id of this RerunWorkflowRequest.  # noqa: E501
        :type: str
        """

        self._re_run_from_workflow_id = re_run_from_workflow_id

    @property
    def workflow_input(self):
        """Gets the workflow_input of this RerunWorkflowRequest.  # noqa: E501


        :return: The workflow_input of this RerunWorkflowRequest.  # noqa: E501
        :rtype: dict(str, object)
        """
        return self._workflow_input

    @workflow_input.setter
    def workflow_input(self, workflow_input):
        """Sets the workflow_input of this RerunWorkflowRequest.


        :param workflow_input: The workflow_input of this RerunWorkflowRequest.  # noqa: E501
        :type: dict(str, object)
        """

        self._workflow_input = workflow_input

    @property
    def re_run_from_task_id(self):
        """Gets the re_run_from_task_id of this RerunWorkflowRequest.  # noqa: E501


        :return: The re_run_from_task_id of this RerunWorkflowRequest.  # noqa: E501
        :rtype: str
        """
        return self._re_run_from_task_id

    @re_run_from_task_id.setter
    def re_run_from_task_id(self, re_run_from_task_id):
        """Sets the re_run_from_task_id of this RerunWorkflowRequest.


        :param re_run_from_task_id: The re_run_from_task_id of this RerunWorkflowRequest.  # noqa: E501
        :type: str
        """

        self._re_run_from_task_id = re_run_from_task_id

    @property
    def task_input(self):
        """Gets the task_input of this RerunWorkflowRequest.  # noqa: E501


        :return: The task_input of this RerunWorkflowRequest.  # noqa: E501
        :rtype: dict(str, object)
        """
        return self._task_input

    @task_input.setter
    def task_input(self, task_input):
        """Sets the task_input of this RerunWorkflowRequest.


        :param task_input: The task_input of this RerunWorkflowRequest.  # noqa: E501
        :type: dict(str, object)
        """

        self._task_input = task_input

    @property
    def correlation_id(self):
        """Gets the correlation_id of this RerunWorkflowRequest.  # noqa: E501


        :return: The correlation_id of this RerunWorkflowRequest.  # noqa: E501
        :rtype: str
        """
        return self._correlation_id

    @correlation_id.setter
    def correlation_id(self, correlation_id):
        """Sets the correlation_id of this RerunWorkflowRequest.


        :param correlation_id: The correlation_id of this RerunWorkflowRequest.  # noqa: E501
        :type: str
        """

        self._correlation_id = correlation_id

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(RerunWorkflowRequest, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, RerunWorkflowRequest):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



from conductor.client.configuration.configuration import Configuration
from six.moves.urllib.parse import urlencode
import certifi
import io
import json
import logging
import re
import six
import ssl
import requests


class RESTResponse(io.IOBase):

    def __init__(self, resp):
        self.status = resp.status_code
        self.reason = resp.reason
        self.resp = resp
        self.headers = resp.headers

    def getheaders(self):
        return self.headers


class RESTClientObject(object):
    def request(self, method, url, query_params=None, headers=None,
                body=None, post_params=None, _preload_content=True,
                _request_timeout=None):
        """Perform requests.

        :param method: http request method
        :param url: http request url
        :param query_params: query parameters in the url
        :param headers: http request headers
        :param body: request json body, for `application/json`
        :param post_params: request post parameters,
                            `application/x-www-form-urlencoded`
                            and `multipart/form-data`
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        """
        method = method.upper()
        assert method in ['GET', 'HEAD', 'DELETE', 'POST', 'PUT',
                          'PATCH', 'OPTIONS']

        if post_params and body:
            raise ValueError(
                "body parameter cannot be used with post_params parameter."
            )

        post_params = post_params or {}
        headers = headers or {}

        timeout = _request_timeout if _request_timeout is not None else 45

        if 'Content-Type' not in headers:
            headers['Content-Type'] = 'application/json'

        try:
            # For `POST`, `PUT`, `PATCH`, `OPTIONS`, `DELETE`
            if method in ['POST', 'PUT', 'PATCH', 'OPTIONS', 'DELETE']:
                if query_params:
                    url += '?' + urlencode(query_params)
                if re.search('json', headers['Content-Type'], re.IGNORECASE) or isinstance(body, str):
                    request_body = '{}'
                    if body is not None:
                        request_body = json.dumps(body)
                    r = requests.request(
                        method, url,
                        data=request_body,
                        timeout=timeout,
                        headers=headers
                    )
                else:
                    # Cannot generate the request from given parameters
                    msg = """Cannot prepare a request message for provided
                             arguments. Please check that your arguments match
                             declared content type."""
                    raise ApiException(status=0, reason=msg)
            # For `GET`, `HEAD`
            else:
                r = requests.request(
                    method, url,
                    params=query_params,
                    timeout=timeout,
                    headers=headers
                )
        except Exception as e:
            msg = "{0}\n{1}".format(type(e).__name__, str(e))
            raise ApiException(status=0, reason=msg)

        if _preload_content:
            r = RESTResponse(r)

        if not 200 <= r.status <= 299:
            raise ApiException(http_resp=r)

        return r

    def GET(self, url, headers=None, query_params=None, _preload_content=True,
            _request_timeout=None):
        return self.request("GET", url,
                            headers=headers,
                            _preload_content=_preload_content,
                            _request_timeout=_request_timeout,
                            query_params=query_params)

    def HEAD(self, url, headers=None, query_params=None, _preload_content=True,
             _request_timeout=None):
        return self.request("HEAD", url,
                            headers=headers,
                            _preload_content=_preload_content,
                            _request_timeout=_request_timeout,
                            query_params=query_params)

    def OPTIONS(self, url, headers=None, query_params=None, post_params=None,
                body=None, _preload_content=True, _request_timeout=None):
        return self.request("OPTIONS", url,
                            headers=headers,
                            query_params=query_params,
                            post_params=post_params,
                            _preload_content=_preload_content,
                            _request_timeout=_request_timeout,
                            body=body)

    def DELETE(self, url, headers=None, query_params=None, body=None,
               _preload_content=True, _request_timeout=None):
        return self.request("DELETE", url,
                            headers=headers,
                            query_params=query_params,
                            _preload_content=_preload_content,
                            _request_timeout=_request_timeout,
                            body=body)

    def POST(self, url, headers=None, query_params=None, post_params=None,
             body=None, _preload_content=True, _request_timeout=None):
        return self.request("POST", url,
                            headers=headers,
                            query_params=query_params,
                            post_params=post_params,
                            _preload_content=_preload_content,
                            _request_timeout=_request_timeout,
                            body=body)

    def PUT(self, url, headers=None, query_params=None, post_params=None,
            body=None, _preload_content=True, _request_timeout=None):
        return self.request("PUT", url,
                            headers=headers,
                            query_params=query_params,
                            post_params=post_params,
                            _preload_content=_preload_content,
                            _request_timeout=_request_timeout,
                            body=body)

    def PATCH(self, url, headers=None, query_params=None, post_params=None,
              body=None, _preload_content=True, _request_timeout=None):
        return self.request("PATCH", url,
                            headers=headers,
                            query_params=query_params,
                            post_params=post_params,
                            _preload_content=_preload_content,
                            _request_timeout=_request_timeout,
                            body=body)


class ApiException(Exception):

    def __init__(self, status=None, reason=None, http_resp=None):
        if http_resp:
            self.status = http_resp.status
            self.reason = http_resp.reason
            self.body = http_resp.resp.text
            self.headers = http_resp.getheaders()
        else:
            self.status = status
            self.reason = reason
            self.body = None
            self.headers = None

    def __str__(self):
        """Custom error messages for exception"""
        error_message = "({0})\n"\
                        "Reason: {1}\n".format(self.status, self.reason)
        if self.headers:
            error_message += "HTTP response headers: {0}\n".format(
                self.headers)

        if self.body:
            error_message += "HTTP response body: {0}\n".format(self.body)

        return error_message
